#include "oh_port.h"
#include "oh_args.h"
#include "oh_debug.h"
#include "oh_tstcs.h"
#include "oh_tstkt.h"

static oh_tk_opt_t _oh_tk_opt =
  {
    OH_NULL,
    "tmp/test",
    1000000UL,
    70000UL
  };

static oh_tk_tc_t _oh_tk_list[] =
  {
    {
      "compiler",
      "oh_tk_testCompiler",
      &oh_tk_testCompiler
    },
    {
      "clib",
      "oh_tk_testCLib",
      &oh_tk_testCLib
    },
    {
      "fs",
      "oh_tk_testFS",
      &oh_tk_testFS
    },
    {
      "args",
      "oh_tk_testArgs",
      &oh_tk_testArgs
    }
  };
static const oh_int _oh_tk_numList = sizeof(_oh_tk_list) / sizeof(oh_tk_tc_t);

oh_int oh_tk_main(oh_int argc, oh_char **argv)
{
  oh_as_opts_t *opts = OH_NULL;
  oh_as_opts_t *test_opts;
  oh_as_opts_t *global_opts;
  oh_int ret = 0;
  oh_long fails = 0;
  oh_int i, j, numtests;

  oh_db_log((OL_DBG, OD_TESTKIT, "oh_tk_main begin"));

  opts = oh_as_parseArgs("-t*:", argv, &argc);
  if (opts == OH_NULL)
  {
    ret = -1;
    goto cleanupret;
  }

  test_opts = oh_as_findOpt(opts, "-t");
  if (test_opts == OH_NULL)
  {
    ret = -1;
    goto cleanupret;
  }

  global_opts = oh_as_getGlobalOpts();
  _oh_tk_opt.homedir = oh_as_findOpt(global_opts, "-h")->opts[1];

  for (i = 1, numtests = test_opts->nopts; i < numtests; i++)
  {
    oh_int sfails;

    for (j = 0; j < _oh_tk_numList; j++)
    {
      if (oh_c_strcmp(test_opts->opts[i], _oh_tk_list[j].explain) == 0)
      {
        oh_db_log((OL_MSG, OD_TESTKIT, "%s (Call %s)", _oh_tk_list[j].explain,
          _oh_tk_list[j].tcfuncname));
        sfails = (*_oh_tk_list[j].tcfunc)(&_oh_tk_opt);
        oh_db_log((OL_MSG, OD_TESTKIT, "%s %d fails",
          _oh_tk_list[j].tcfuncname, sfails));
        fails += sfails;
        OH_ASSERT(fails >= 0); /* overflow check */
        goto nextopt; /* use "goto" to go out from internal for statement */
      }
    }

    if (oh_c_strcmp(test_opts->opts[i], "all") == 0)
    {
      for (j = 0; j < _oh_tk_numList; j++)
      {
        oh_db_log((OL_MSG, OD_TESTKIT, "%s (Call %s)", _oh_tk_list[j].explain,
          _oh_tk_list[j].tcfuncname));
        sfails = (*_oh_tk_list[j].tcfunc)(&_oh_tk_opt);
        oh_db_log((OL_MSG, OD_TESTKIT, "%s %d fails",
          _oh_tk_list[j].tcfuncname, sfails));
        fails += sfails;
        OH_ASSERT(fails >= 0); /* overflow check */
      }
      break;
    }
    else /* if there is an unknown test case argument */
    {
      ret = -1;
      goto cleanupret;
    }
nextopt:
    ;
  }

cleanupret:
  oh_as_freeOpts(opts);

  /* return < 0 only if fail to parse "-t" argument */
  if (ret == 0)
  {
    if (fails == 0)
    {
      ret = 0;
    }
    else
    {
      ret = 1;
    }
  }

  oh_db_log((OL_DBG, OD_TESTKIT, "oh_tk_main %ld fails", fails));
  oh_db_log((OL_DBG, OD_TESTKIT, "oh_tk_main end with exit code %d", ret));

  return ret;
}
