
/**
 * @file oh_fsutl.c
 * @version $Id$
 *
 * The source that has dos and unix format file name transformation utilities.
 */

#include "oh_port.h"
#include "oh_fsutl.h"
#include "oh_fs.h"
#include "oh_debug.h"

/**
 * Transforms dos format file name to unix format file name.
 *
 * The path argument points to the character array string. The string has
 * dos format file name before calling. The string has unix format file name
 * after returning. For example, c:\home\ohhara is the dos format file name and
 * /c/home/ohhara is the unix format file name.
 *
 * @param path The char string that is the dos format file name that will be
 * changed to the unix format file name.
 * @return none
 */
oh_void oh_fu_dos2unix(oh_char *path)
{
  if (path[1] == ':')
  {
    path[1] = path[0];
    path[0] = '/';
    path += 2;
  }

  for (; *path != '\0'; path++)
  {
    if (*path == '\\')
      *path = '/';
  }
}

/**
 * Transforms unix format file name to dos format file name.
 *
 * The path argument points to the character array string. The string has
 * unix format file name before calling. The string has dos format file name
 * after returning. For example, /c/home/ohhara is the unix format file name
 * and c:\home\ohhara is the dos format file name.
 *
 * @param path The char string that is the unix format file name that will be
 * changed to the dos format file name.
 * @return none
 */
oh_void oh_fu_unix2dos(oh_char *path)
{
  if (path[0] == '/' && path[2] == '/')
  {
    path[0] = path[1];
    path[1] = ':';
    path += 2;
  }

  for (; *path != '\0'; path++)
  {
    if (*path == '/')
      *path = '\\';
  }
}
